#include <konkret/konkret.h>
#include "Capabilities.h"
#include "Resource.h"

static const CMPIBroker* _cb = NULL;

static void CapabilitiesInitialize()
{
}

static CMPIStatus CapabilitiesCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus CapabilitiesEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus CapabilitiesEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Capabilities x;

    Capabilities_Init(&x, _cb, KNameSpace(cop));

    /* InstanceID */
    Capabilities_Set_InstanceID(&x, "SBLIM:Linux_SSHCapabilities");

    /* ElementName */
    Capabilities_Set_ElementName(&x, "Linux_SSHCapabilities");

    /* RequestedStatesSupported */
    Capabilities_Init_RequestedStatesSupported(&x, 0);

    /* ElementNameEditSupported */
    Capabilities_Set_ElementNameEditSupported(&x, 0);

    /* MaxElementNameLen */
    Capabilities_Set_MaxElementNameLen(&x, 65535);

    /* MaxConnections */
    Capabilities_Set_MaxConnections(&x, 65535);

    /* SupportedSSHVersions */
    {
        struct SSHDConfig conf;
        CMPICount index = 0;

        if (GetSSHDConfig(&conf) != 0)
            KReturn2(_cb, ERR_FAILED, "failed to access sshd_config");

        Capabilities_Init_SupportedSSHVersions(&x, conf.v1 + conf.v2);

        if (conf.v1)
            Capabilities_Set_SupportedSSHVersions_SSHv1(&x, index++);

        if (conf.v2)
            Capabilities_Set_SupportedSSHVersions_SSHv2(&x, index++);
    }

    /* SupportedEncryptionAlgorithms */
    Capabilities_Init_SupportedEncryptionAlgorithms(&x, 1);
    Capabilities_Set_SupportedEncryptionAlgorithms_DES3(&x, 0);

    /* ListeningPortManagementSupported */
    Capabilities_Set_ListeningPortManagementSupported(&x, 0);

    /* MaxListeningPorts */
    Capabilities_Set_MaxListeningPorts(&x, 65535);

    KReturnInstance(cr, x);

    KReturn(OK);
}

static CMPIStatus CapabilitiesGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus CapabilitiesCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus CapabilitiesModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus CapabilitiesDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus CapabilitiesExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Capabilities, 
    Linux_SSHCapabilities, 
    _cb, 
    CapabilitiesInitialize())

static CMPIStatus CapabilitiesMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus CapabilitiesInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Capabilities_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Capabilities,
    Linux_SSHCapabilities,
    _cb,
    CapabilitiesInitialize())

KUint16 Capabilities_CreateGoalSettings(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const CapabilitiesRef* self,
    const KStringA* TemplateGoalSettings,
    KStringA* SupportedGoalSettings,
    CMPIStatus* status)
{
    KUint16 result = KUINT16_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_SSHCapabilities",
    "Linux_SSHCapabilities",
    "instance method")
