#include <konkret/konkret.h>
#include "ElementCapabilities.h"
#include "Resource.h"

static const CMPIBroker* _cb;

static void ElementCapabilitiesInitialize()
{
}

static CMPIStatus ElementCapabilitiesCleanup( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ElementCapabilitiesEnumInstanceNames( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus ElementCapabilitiesEnumInstances( 
    CMPIInstanceMI* mi,
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    const char* ns = KNameSpace(cop);
    ProtocolServiceRef m; /* ManagedElement */
    CapabilitiesRef c; /* Capabilities */
    ElementCapabilities x;
    Str256 s;

    /* Create ElementCapabilities.ManagedElement */
    ProtocolServiceRef_Init(&m, _cb, ns);
    ProtocolServiceRef_Set_SystemCreationClassName(&m, SysClassName(&s));
    ProtocolServiceRef_Set_CreationClassName(&m, "Linux_SSHProtocolService");
    ProtocolServiceRef_Set_SystemName(&m, SysName(&s));
    ProtocolServiceRef_Set_Name(&m, "SBLIM:Linux_SSHProtocolService");

    /* Create ElementCapabilities.Capabilities */
    CapabilitiesRef_Init(&c, _cb, ns);
    CapabilitiesRef_Set_InstanceID(&c, "SBLIM:Linux_SSHCapabilities");

    /* Create ElementCapabilities */
    ElementCapabilities_Init(&x, _cb, ns);
    ElementCapabilities_Set_ManagedElement(&x, &m);
    ElementCapabilities_Set_Capabilities(&x, &c);
    KReturnInstance(cr, x);

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ElementCapabilitiesGetInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc,
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char** properties) 
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus ElementCapabilitiesCreateInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const CMPIInstance* ci) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ElementCapabilitiesModifyInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop,
    const CMPIInstance* ci, 
    const char**properties) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ElementCapabilitiesDeleteInstance( 
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ElementCapabilitiesExecQuery(
    CMPIInstanceMI* mi, 
    const CMPIContext* cc, 
    const CMPIResult* cr, 
    const CMPIObjectPath* cop, 
    const char* lang, 
    const char* query) 
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ElementCapabilitiesAssociationCleanup( 
    CMPIAssociationMI* mi,
    const CMPIContext* cc, 
    CMPIBoolean term) 
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ElementCapabilitiesAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        ElementCapabilities_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus ElementCapabilitiesAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        ElementCapabilities_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus ElementCapabilitiesReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        ElementCapabilities_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus ElementCapabilitiesReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        ElementCapabilities_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub( 
    ElementCapabilities,
    Linux_SSHElementCapabilities,
    _cb,
    ElementCapabilitiesInitialize())

CMAssociationMIStub( 
    ElementCapabilities,
    Linux_SSHElementCapabilities,
    _cb,
    ElementCapabilitiesInitialize())

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_SSHElementCapabilities",
    "Linux_SSHElementCapabilities",
    "instance association")
