#include <konkret/konkret.h>
#include "ProtocolEndpoint.h"
#include "Resource.h"

static const CMPIBroker* _cb = NULL;

static void ProtocolEndpointInitialize()
{
}

static CMPIStatus ProtocolEndpointCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ProtocolEndpointEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static char* _NameOf(Str256* s, struct SSHSession* sess)
{
    sprintf(s->buf, "SBLIM:Linux_SSHProtocolEndpoint:%u", sess->localPort);
    return s->buf;
}

static CMPIStatus ProtocolEndpointEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    const char* ns = KNameSpace(cop);
    struct SSHSession* sess;
    size_t size;
    size_t i;

    /* Disocover SSH sessions */

    if (GetSSHSessions(&sess, &size) != 0)
        KReturn2(_cb, ERR_FAILED, "SSH session discovery failure (netstat)");

    /* For each SSH session */

    for (i = 0; i < size; i++)
    {
        ProtocolEndpoint x;
        struct SSHDConfig conf;
        Str256 s;

        if (GetSSHDConfig(&conf) != 0)
            KReturn2(_cb, ERR_FAILED, "failed to access sshd_config");

        ProtocolEndpoint_Init(&x, _cb, ns);

        /* Linux_SSHProtocolEndpoint.SystemCreationClassName */
        ProtocolEndpoint_Set_SystemCreationClassName(&x, SysClassName(&s));

        /* Linux_SSHProtocolEndpoint.CreationClassName */
        ProtocolEndpoint_Set_CreationClassName(&x, "Linux_SSHProtocolEndpoint");

        /* Linux_SSHProtocolEndpoint.SystemName */
        ProtocolEndpoint_Set_SystemName(&x, SysName(&s));

        /* Linux_SSHProtocolEndpoint.Name */
        ProtocolEndpoint_Set_Name(&x, _NameOf(&s, &sess[i]));

        /* Linux_SSHProtocolEndpoint.NameFormat */
        ProtocolEndpoint_Set_NameFormat(&x,
            "'SBLIM:' + <CreationClassName> + ':' + <Port>");

        /* Linux_SSHProtocolEndpoint.ProtocolIFType */
        ProtocolEndpoint_Set_ProtocolIFType_SSH(&x);

        /* Linux_SSHProtocolEndpoint.OtherTypeDescription */
        ProtocolEndpoint_Set_OtherTypeDescription(&x, "SSH");

        /* Linux_SSHProtocolEndpoint.ElementName */
        ProtocolEndpoint_Set_ElementName(&x, "Linux_SSHProtocolEndpoint");

        /* Linux_SSHProtocolEndpoint.EnabledSSHVersions */
        {
            CMPICount index = 0;

            ProtocolEndpoint_Init_EnabledSSHVersions(&x, conf.v1 + conf.v2);

            if (conf.v1)
                ProtocolEndpoint_Set_EnabledSSHVersions_SSHv1(&x, index++);

            if (conf.v2)
                ProtocolEndpoint_Set_EnabledSSHVersions_SSHv2(&x, index++);
        }

        /* Linux_SSHProtocolEndpoint.SSHVersion */
        if (conf.v1 && conf.v2)
            ProtocolEndpoint_Set_SSHVersion_Unknown(&x);
        else if (conf.v1)
            ProtocolEndpoint_Set_SSHVersion_SSHv1(&x);
        else if (conf.v2)
            ProtocolEndpoint_Set_SSHVersion_SSHv2(&x);

        /* Linux_SSHProtocolEndpoint.EnabledEncryptionAlgorithms (ATTN) */
        ProtocolEndpoint_Init_EnabledEncryptionAlgorithms(&x, 1);
        ProtocolEndpoint_Set_EnabledEncryptionAlgorithms_Unknown(&x, 0);

        /* Linux_SSHProtocolEndpoint.EncryptionAlgorithm (ATTN) */
        ProtocolEndpoint_Set_EncryptionAlgorithm_Unknown(&x);

        /* Linux_SSHProtocolEndpoint.IdleTimeout (ATTN) */
        ProtocolEndpoint_Set_IdleTimeout(&x, 0);

        /* Linux_SSHProtocolEndpoint.KeepAlive */
        ProtocolEndpoint_Set_KeepAlive(&x, conf.tcpKeepAlive);

        /* Linux_SSHProtocolEndpoint.x11Forwarding */
        ProtocolEndpoint_Set_ForwardX11(&x, conf.x11Forwarding);

        /* Linux_SSHProtocolEndpoint.Compression */
        if (conf.compression == 'y' || conf.compression == 'd')
            ProtocolEndpoint_Set_Compression(&x, 1);
        else ProtocolEndpoint_Set_Compression(&x, 0);

        /* Linux_SSHProtocolEndpoint.LocalHost (Extra) */
        ProtocolEndpoint_Set_LocalHost(&x, sess[i].localAddr);

        /* Linux_SSHProtocolEndpoint.LocalPort (Extra) */
        ProtocolEndpoint_Set_LocalPort(&x, sess[i].localPort);

        /* Linux_SSHProtocolEndpoint.RemoteHost (Extra) */
        ProtocolEndpoint_Set_RemoteHost(&x, sess[i].foreignAddr);

        /* Linux_SSHProtocolEndpoint.RemotePort (Extra) */
        ProtocolEndpoint_Set_RemotePort(&x, sess[i].foreignPort);

        /* Linux_SSHProtocolEndpoint.AuthenticatedUser (Extra) */
        ProtocolEndpoint_Set_AuthenticatedUser(&x, sess[i].user);

        KReturnInstance(cr, x);
    }

    free(sess);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ProtocolEndpointGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus ProtocolEndpointCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ProtocolEndpointModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ProtocolEndpointDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ProtocolEndpointExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    ProtocolEndpoint, 
    Linux_SSHProtocolEndpoint, 
    _cb, 
    ProtocolEndpointInitialize())

static CMPIStatus ProtocolEndpointMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ProtocolEndpointInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return ProtocolEndpoint_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    ProtocolEndpoint,
    Linux_SSHProtocolEndpoint,
    _cb,
    ProtocolEndpointInitialize())

KUint32 ProtocolEndpoint_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolEndpointRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 ProtocolEndpoint_BroadcastReset(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolEndpointRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_SSHProtocolEndpoint",
    "Linux_SSHProtocolEndpoint",
    "instance method")
