#include <konkret/konkret.h>
#include <cmpiutil/base.h>
#include <cmpiutil/exec.h>
#include "ProtocolService.h"
#include "Resource.h"

static const CMPIBroker* _cb = NULL;

static void ProtocolServiceInitialize()
{
}

static CMPIStatus ProtocolServiceCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ProtocolServiceEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus ProtocolServiceEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    ProtocolService x;
    int sshd_status;
    Str256 s;

    /* Get status of sshd */
    sshd_status = GetSSHDStatus();

    ProtocolService_Init(&x, _cb, KNameSpace(cop));

    /* SystemCreationClassName */
    ProtocolService_Set_SystemCreationClassName(&x, SysClassName(&s));

    /* CreationClassName */
    ProtocolService_Set_CreationClassName(&x, "Linux_SSHProtocolService");

    /* SystemName */
    ProtocolService_Set_SystemName(&x, SysName(&s));

    /* Name */
    ProtocolService_Set_Name(&x, "SBLIM:Linux_SSHProtocolService");

    /* Protocol */
    ProtocolService_Set_Protocol_SSH(&x);

    /* MaxConnections */
    ProtocolService_Set_MaxConnections(&x, 65535);

    /* EnabledState */
    if (sshd_status == 0)
        ProtocolService_Set_EnabledState_Enabled(&x);
    else
        ProtocolService_Set_EnabledState_Disabled(&x);

    /* ElementName */
    ProtocolService_Set_ElementName(&x, "Linux_SSHProtocolService");

    /* OperationalStatus */
    ProtocolService_Init_OperationalStatus(&x, 1);

    if (sshd_status == 0)
        ProtocolService_Set_OperationalStatus_OK(&x, 0);
    else
        ProtocolService_Set_OperationalStatus_Unknown(&x, 0);

    /* HealthState */
    if (sshd_status == 0)
        ProtocolService_Set_HealthState_OK(&x);
    else
        ProtocolService_Set_HealthState_Unknown(&x);

    KReturnInstance(cr, x);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ProtocolServiceGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus ProtocolServiceCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ProtocolServiceModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ProtocolServiceDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus ProtocolServiceExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    ProtocolService, 
    Linux_SSHProtocolService, 
    _cb, 
    ProtocolServiceInitialize())

static CMPIStatus ProtocolServiceMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus ProtocolServiceInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return ProtocolService_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    ProtocolService,
    Linux_SSHProtocolService,
    _cb,
    ProtocolServiceInitialize())

KUint32 ProtocolService_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolServiceRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 ProtocolService_StartService(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolServiceRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 ProtocolService_StopService(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolServiceRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 ProtocolService_ChangeAffectedElementsAssignedSequence(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolServiceRef* self,
    const KRefA* ManagedElements,
    const KUint16A* AssignedSequence,
    KRef* Job,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 ProtocolService_ListenOnPort(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const ProtocolServiceRef* self,
    const KUint32* PortNumber,
    const KRef* IPEndpoint,
    KRef* TCPEndpoint,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "Linux_SSHProtocolService",
    "Linux_SSHProtocolService",
    "instance method")
