#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <cmpiutil/base.h>
#include <cmpiutil/exec.h>
#include "Resource.h"

/* ATTN: Figure out why this is not visible from <strings.h> */
extern char *strcasestr(const char *haystack, const char *needle);

int GetSSHDStatus()
{
#if 0
    /* This version does not suppress standard output */
    char* cmd[] = { "/etc/init.d/sshd", "status", NULL };
    return cmpiutilSafeSystem(cmd, NULL) == 0 ? 0 : -1;
#else
    FILE* is;
    char buf[256];

    if (!(is = popen("/etc/init.d/sshd status", "r")))
        return -1;

    if (fgets(buf, sizeof(buf), is) == NULL)
    {
        pclose(is);
        return -1;
    }

    if (strcasestr(buf, "running") == NULL)
    {
        pclose(is);
        return -1;
    }

    pclose(is);
    return 0;
#endif
}

int GetSSHDConfig(struct SSHDConfig* conf)
{
    FILE* is;
    char buf[256];

    memset(conf, 0, sizeof(struct SSHDConfig));

    /* Open sshd_config */

    if (!conf || !(is = fopen("/etc/ssh/sshd_config", "r")))
        return -1;

    /* Set defaults */

    conf->v1 = 1;
    conf->v2 = 1;
    conf->tcpKeepAlive = 1;
    conf->compression = 'd';
    conf->x11Forwarding = 0;
    conf->clientAliveInterval = 0;

    /* For each line of configuration file */

    while (fgets(buf, sizeof(buf), is) != NULL)
    {
        if (strncmp(buf, "Protocol", 8) == 0 && isspace(buf[8]))
        {
            conf->v1 = 0;
            conf->v2 = 0;

            if (strchr(buf, '1'))
                conf->v1 = 1;
            if (strchr(buf, '2'))
                conf->v2 = 1;
        }
        else if (strncmp(buf, "TCPKeepAlive", 12) == 0 && isspace(buf[12]))
        {
            conf->tcpKeepAlive = strcasestr(buf, "yes") ? 1 : 0;
        }
        else if (strncmp(buf, "X11Forwarding", 10) == 0 && isspace(buf[10]))
        {
            conf->x11Forwarding = strcasestr(buf, "yes") ? 1 : 0;
        }
        else if (strncmp(buf, "Compression", 11) == 0 && isspace(buf[11]))
        {
            if (strcasestr(buf, "yes"))
                conf->compression = 'y';
            else if (strcasestr(buf, "no"))
                conf->compression = 'n';
            else if (strcasestr(buf, "delayed"))
                conf->compression = 'd';
        }
        else if (strncmp(buf, "ClientAliveInterval", 19) == 0 && 
            isspace(buf[19]))
        {
            const char* p = &buf[19];

            while (isspace(*p))
                p++;

            conf->clientAliveInterval = atoi(p);
        }
    }

    /* Close file */
    fclose(is);

    return 0;
}

#if 0
static void _dumpSSHSession(const struct SSHSession* sess)
{
    printf("==== SSHSession\n");
    printf("localAddr{%s}\n", sess->localAddr);
    printf("localPort{%u}\n", sess->localPort);
    printf("foreignAddr{%s}\n", sess->foreignAddr);
    printf("foreignPort{%u}\n", sess->foreignPort);
    printf("pid{%u}\n", sess->pid);
    printf("program{%s}\n", sess->program);
}
#endif

int GetSSHSessions(
    struct SSHSession** sessions,
    size_t* num_sessions)
{
    FILE* is;
    char buf[256];
    struct SSHSession* data = NULL;
    size_t size = 0;

    if (!sessions || !num_sessions)
        return -1;

    *sessions = NULL;
    *num_sessions = 0;

    /* 
     * Run netstat
     *   -t -- tcp
     *   -n -- show numerical addresses
     *   -p show pid and nameo of program
     */

    if (!(is = popen("/bin/netstat -tnp 2> /dev/null", "r")))
        return -1;

    /* For each line of netstat output */

    while (fgets(buf, sizeof(buf), is) != NULL)
    {
        size_t n = strlen(buf);
        char* end = buf + n;
        struct SSHSession sess;
        struct SSHSession* ptr;

        memset(&sess, 0, sizeof(sess));

        /* Remove trailing space */

        while (end != buf && isspace(end[-1]))
            *--end = '\0';

        /* Skip first line (title header). Ordinary lines start with "tcp" */

        if (buf[0] != 't')
            continue;

        /* Skip lines not containing "/sshd:" */

        if (strstr(buf, "/sshd:") == NULL)
            continue;

        /* Extract columns (do not split around ':' due to IPV6 addresses) */
        {
            char* s;
            char* p;
            size_t i = 0;

            for (p = strtok_r(buf, " \t", &s); p; p = strtok_r(NULL, " \t", &s))
            {
                char* q;

                switch (i)
                {
                    /* Proto */
                    case 0: 
                    /* Recv-Q */
                    case 1: 
                    /* Send-Q */
                    case 2: 
                        break;
                    /* Local Address */
                    case 3: 
                    {
                        if (!(q = strrchr(p, ':')))
                            continue;

                        *q = '\0';
                        strncat(sess.localAddr, p, sizeof(sess.localAddr)-1);
                        sess.localPort = atoi(q + 1);
                        break;
                    }
                    /* Foreign Address */
                    case 4: 
                    {
                        if (!(q = strrchr(p, ':')))
                            continue;

                        *q = '\0';
                        strncat(sess.foreignAddr, p,sizeof(sess.foreignAddr)-1);
                        sess.foreignPort = atoi(q + 1);
                        break;
                    }
                    /* State */
                    case 5: 
                    {
                        break;
                    }
                    /* PID/Program name */
                    case 6: 
                    {
                        if (!(q = strrchr(p, '/')))
                            continue;

                        *q = '\0';
                        sess.pid = atoi(p);
                        break;
                    }
                    /* Username */
                    case 7:
                    {
                        *sess.user = '\0';
                        strncat(sess.user, p, sizeof(sess.user) - 1);
                        break;
                    }
                }

                i++;
            }
        }

        /* Append session to array */

        ptr = (struct SSHSession*)realloc(data, (size + 1) * sizeof(sess));

        if (!ptr)
        {
            free(data);
            pclose(is);
            return -1;
        }

        data = ptr;
        memcpy(&data[size++], &sess, sizeof(sess));
    }

    pclose(is);
    *sessions = data;
    *num_sessions = size;
    return 0;
}
